/*
 * LinearScaler.java
 *
 * Created on July 24, 2006, 9:40 AM
 *
 */
package conversion;

/**
 *  A LinearScaler is a scaler that uses a linear conversion
 *  formula.
 *  It is parametrized by a zero point and a slope and scales the value val
 *  to zero_point + slope*val.
 *
 *  @author gshute
 */
public class LinearScaler
        implements Scaler {

  private class Parameters {

    private double zeroPoint;
    private double slope;

    private void setScale(double z, double s) {
      zeroPoint = z;
      slope = s;
    }

    private void setScale(double z, double val, double sval) {
      zeroPoint = z;
      slope = (sval - z) / val;
    }

    private void setScale(double val1, double sval1, double val2, double sval2) {
      slope = (sval2 - sval1) / (val2 - val1);
      zeroPoint = sval1 - slope * val1;
    }

  }

  private Parameters parameters;
  private Scaler reverseScaler;

  /**
   *  doSetScale(val) adjusts this linear scaler to have zero point z and slope s.
   * @param z 
   * @param s 
   */
  protected void doSetScale(double z, double s) {
    parameters.setScale(z, s);
  }

  /**
   *  doSetScale(val) adjusts this linear scaler to have zero point z and to
   *  scale val to sval.
   * @param z 
   * @param val 
   * @param sval 
   */
  protected void doSetScale(double z, double val, double sval) {
    parameters.setScale(z, val, sval);
  }

  /**
   *  doSetScale(val) adjusts this linear scaler to scale val1 to sval1 and
   *  val2 to sval2.
   * @param val1 
   * @param sval1 
   * @param val2 
   * @param sval2 
   */
  protected void doSetScale(double val1, double sval1, double val2, double sval2) {
    parameters.setScale(val1, sval1, val2, sval2);
  }

  /**
   *  new LinearScaler() returns an identity scaler.
   *  That is, both scale() and reverseScale() return their
   *  argument values unchanged.
   */
  public LinearScaler() {
    this(0.0, 1.0);
  }

  /**
   *  new LinearScaler(z, s) returns a new LinearScaler with
   *  zeroPoint z and slope s.
   * @param z 
   * @param s 
   */
  public LinearScaler(double z, double s) {
    parameters = new Parameters();
    doSetScale(z, s);
  }

  /**
   *  new LinearScaler(z, val, sval) a new LinearScaler with
   *  zeroPoint z that scales val to sval.
   * @param z 
   * @param val 
   * @param sval 
   */
  public LinearScaler(double z, double val, double sval) {
    parameters = new Parameters();
    doSetScale(z, val, sval);
  }

  /**
   *  new LinearScaler(val1, sval1, val2, sval2) returns a new
   *  LinearScaler that scales val1 to sval1 and val2 to sval2.
   * @param val1 
   * @param sval1 
   * @param val2 
   * @param sval2 
   */
  public LinearScaler(double val1, double sval1, double val2, double sval2) {
    parameters = new Parameters();
    doSetScale(val1, sval1, val2, sval2);
  }

  /**
   *  ls.scale(val) returns ls's scaled value for val.
   * @param val 
   * @return 
   */
  public double scale(double val) {
    return parameters.zeroPoint + parameters.slope * val;
  }

  /**
   *  ls.getReverseScaler() returns a scaler that performs
   *  scaling that is inverse to ls' scaling.
   *  The returned scaler tracks changes in parameters that
   *  control ls's scaling.
   * @return 
   */
  public Scaler getReverseScaler() {
    if (reverseScaler == null) {
      reverseScaler = new Scaler() {

        public double scale(double val) {
          return (val - parameters.zeroPoint) / parameters.slope;
        }

        public Scaler getReverseScaler() {
          return LinearScaler.this;
        }

      };
    }
    return reverseScaler;
  }

}  // public class LinearScaler
