package gui;

import java.awt.Dimension;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import javax.swing.JComponent;

/**
 * The AbstractSizer class is the base class for classes
 * that manage resizing components such as text panes
 * that can lay out their contents differently depending
 * on available space.
 * <p>
 * The setWidth() and setHeight() methods are TemplateMethods
 * in a Template Method design pattern, along with a private
 * method that is invoked whenever the sized component is
 * sized by a layout manager.
 * Subclasses complete the implementation by implementing
 * the primitive operation computePreferredSize().
 * </p>
 * <p>
 * An AbstractSizer variable may appear to be unused.
 * This is because its methods are not always invoked directly.
 * They may be only invoked through the ComponentListener that
 * it installs on its component.
 *
 * @author gshute
 */
public abstract class AbstractSizer {

  private JComponent component;
  private Dimension lastActualSize;

  private void checkSize() {
    Dimension actualSize = component.getSize();
    Dimension computedSize = computePreferredSize(actualSize);
    if (!actualSize.equals(computedSize) &&
            !actualSize.equals(lastActualSize)) {
      component.setPreferredSize(computedSize);
      component.setMinimumSize(computedSize);
      component.setMaximumSize(computedSize);
    }
    lastActualSize = actualSize;
  }
  
  /**
   * computePreferredSize(sz) returns a preferred size
   * based on the current actual size sz.
   * Most implementations should return a new size with
   * only one of the dimensions altered.
   * 
   * @return the new preferred size for the component.
   */
  protected abstract Dimension computePreferredSize(Dimension sz);

  /**
   * new AbstractSizer(c) returns a new sizer whosed
   * sized component is c.
   * 
   * @param c the component to be sized
   */
  public AbstractSizer(JComponent c) {
    component = c;
    lastActualSize = component.getSize();
    component.addComponentListener(new ComponentAdapter() {

      public void componentResized(ComponentEvent ev) {
        checkSize();
      }

    });
  }

  /**
   * s.setWidth(w) sets the initial width of the sized
   * component of s to w.
   * The initial height will be determined by the
   * computePreferredSize() method.
   * 
   * @param w the initial width
   */
  public void setWidth(int w) {
    component.setSize(new Dimension(w, 2000));
    checkSize();
  }

  /**
   * s.setHeight(h) sets the initial heighth of the sized
   * component of s to h.
   * The initial width will be determined by the
   * computePreferredSize() method.
   * 
   * @param h the initial width
   */
  public void setHeight(int h) {
    component.setSize(new Dimension(2000, h));
    checkSize();
  }

}  //public abstract class AbstractSizer
