package gui;

import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPopupMenu;

/**
 * An AttachablePopupMenu is a popup menu that can be
 * attached to several components at the same time.
 * Menu items can be added using the JPopupMenu
 * interface.
 */
public class AttachablePopupMenu
        extends JPopupMenu {

  /**
   * popUpOnTrigger(c, ev) pops up this popup menu for c
   * if ev is a popup trigger.
   */
  private void popUpOnTrigger(Component c, MouseEvent ev) {
    if (ev.isPopupTrigger()) {
      show(c, ev.getX(), ev.getY());
    }
  }

  /**
   * new AttachablePopupMenu() returns a new attachable
   * popup menu that has no title.
   */
  public AttachablePopupMenu() {
    super();
  }

  /**
   * new AttachablePopupMenu(ttl) returns a new attachable
   * popup menu with title ttl.
   * 
   * @param ttl the title displayed by the popup menu
   */
  public AttachablePopupMenu(String ttl) {
    super(ttl);
  }

  /**
   * p.attachTo(c) makes c respond to popup trigger events
   * by popping up p.
   * This method can be called more than once with different
   * arguments.
   * The result is that p is popped up whenever a popup
   * trigger is detected on any component that p is attached
   * to.
   * 
   * @param c the component to which p is attached
   */
  public void attachTo(final Component c) {
    c.addMouseListener(new MouseAdapter() {

      public void mousePressed(MouseEvent ev) {
        popUpOnTrigger(c, ev);
      }

      public void mouseClicked(MouseEvent ev) {
        popUpOnTrigger(c, ev);
      }

      public void mouseReleased(MouseEvent ev) {
        popUpOnTrigger(c, ev);
      }

    });
  }

}  // public class AttachablePopupMenu

