package gui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.Action;
import javax.swing.JPanel;

/**
 * The DrawPanel class is a base class for panels
 * that paint themselves using Java2D graphics.
 * <p>
 * It is a simple example of the Template Method
 * design pattern whose TemplateMethods are the
 * methods paintComponent() and getPrintAction().
 * Subclasses complete the implementation by
 * implementing the primitive operation draw(). 
 * </p>
 * 
 * @author gshute
 */
public abstract class DrawPanel
extends JPanel {
  
  private Action printAction;
  private Action exportAction;
  
  /**
   * The draw() method should be implemented by
   * concrete subclasses to send messages to its
   * graphics object argument.
   * Subclasses can draw whatever they want.
   * The drawing is visibly updated when the draw
   * panel receives a repaint() message.
   * 
   * @param g the graphics object
   */
  protected abstract void draw(Graphics2D g);
  
  /**
   * new DrawPanel() returns a draw panel with
   * dimensions 200 by 200.
   */
  public DrawPanel() {
    super();
    setPreferredSize(new Dimension(200, 200));
  }
  
  /**
   * This method is invoked internally.
   * It should not be invoked directly by most
   * clients.
   */
  public final void paintComponent(Graphics g) {
    draw((Graphics2D)g);
  }
  
  /**
   * dp.getPrintAction returns a print action
   * for initiating printing of dp.
   * 
   * @return the print action
   */
  public final Action getPrintAction() {
    if (printAction == null) {
      ComponentGraphics componentGraphics =
              new ComponentGraphics(this);
      printAction = componentGraphics.getPrintAction();
    }
    return printAction;
  }

  /**
   * dp.getPrintAction returns a print action
   * for initiating printing of dp.
   *
   * @return the print action
   */
  public final Action getExportAction() {
    if (exportAction == null) {
      ComponentGraphics componentGraphics =
              new ComponentGraphics(this);
      exportAction = componentGraphics.getExportAction();
    }
    return exportAction;
  }

}  // public abstract class DrawPanel
