package gui;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import javax.swing.JPanel;
import javax.swing.Scrollable;

/**
 * A GriddedPanel lays out the subcomponents that are
 * added to it on a grid.
 * The number of rows and columns in the grid is
 * determined automatically based on its width, the
 * widths of its subcomponents, and the number of
 * subcomponents.
 * <p>
 * The gaps between subcomponents can be set using the
 * setHGap() and setVGap() methods.
 * Gaps between the subcomponents and the boundaries
 * of the gridded panel can be set by setting an empty
 * border for the gridded panel.
 * </p>
 * 
 * @author gshute
 */
public class GriddedPanel
        extends JPanel
        implements Scrollable {

    private AbstractSizer sizer;
    private GridLayout layout;
    private int cellHeight;

    /**
     * computeCellSize() computes the size of a grid cell
     * incuding horizontal and vertical gaps.
     * The cell height is saved in the instance variable
     * cellHeight and the cell width is returned.
     */
    private int computeCellSize() {
        Dimension layoutSize = layout.preferredLayoutSize(this);
        Insets insets = getInsets();
        int rowCount = layout.getRows();
        int layoutHeight = layoutSize.height + layout.getVgap();
        layoutHeight -= insets.top + insets.bottom;
        cellHeight = layoutHeight / rowCount;
        int cellCount = getComponentCount();
        int columnCount = (cellCount + rowCount - 1) / rowCount;
        int layoutWidth = layoutSize.width + layout.getHgap();
        layoutWidth -= insets.left + insets.right;
        return layoutWidth / columnCount;
    }

    /**
     * computeSize(sz) returns a new preferred size based on
     * the current width given in sz.
     */
    private Dimension computeSize(Dimension sz) {
        int cellCount = getComponentCount();
        if (cellCount == 0) {
            return sz;
        }
        int cellWidth = computeCellSize();
        int columns = (sz.width + layout.getHgap()) / cellWidth;
        if (columns == 0) {
            columns = 1;
        }
        int rows = (cellCount + columns - 1) / columns;
        int newHeight = rows * cellHeight - layout.getVgap();
        Insets insets = getInsets();
        newHeight += insets.top + insets.bottom;
        layout.setRows(rows);
        return new Dimension(getWidth(), newHeight);
    }

    public GriddedPanel() {
        layout = new GridLayout(1, 0);
        setLayout(layout);
        sizer = new AbstractSizer(this) {

            public Dimension computePreferredSize(Dimension sz) {
                return computeSize(sz);
            }
        };
    }

    /**
     * This method is invoked by a JScrollPane that
     * contains this GriddedPanel.
     * It does not need to be invoked directly by other
     * clients.
     */
    public boolean getScrollableTracksViewportWidth() {
        return true;
    }

    /**
     * This method is invoked by a JScrollPane that
     * contains this GriddedPanel.
     * It does not need to be invoked directly by other
     * clients.
     */
    public boolean getScrollableTracksViewportHeight() {
        return false;
    }

    /**
     * This method is invoked by a JScrollPane that
     * contains this GriddedPanel.
     * It does not need to be invoked directly by other
     * clients.
     */
    public Dimension getPreferredScrollableViewportSize() {
        return getPreferredSize();
    }

    /**
     * This method is invoked by a JScrollPane that
     * contains this GriddedPanel.
     * It does not need to be invoked directly by other
     * clients.
     */
    public int getScrollableUnitIncrement(Rectangle r, int or, int dir) {
        return cellHeight;
    }

    /**
     * This method is invoked by a JScrollPane that
     * contains this GriddedPanel.
     * It does not need to be invoked directly by other
     * clients.
     */
    public int getScrollableBlockIncrement(Rectangle r, int or, int dir) {
        int cellsToScroll = r.height / cellHeight;
        if (cellsToScroll == 0) {
            cellsToScroll = 1;
        }
        return cellsToScroll * cellHeight;
    }

    /**
     * gp.setHGap(h) sets the horizontal gap between
     * the componets in gp to h.
     *
     * @param h the horizontal gap between components
     */
    public void setHGap(int h) {
        layout.setHgap(h);
    }

    /**
     * gp.setVGap(v) sets the vertical gap between
     * the componets in gp to v.
     *
     * @param v the vertical gap between components
     */
    public void setVGap(int v) {
        layout.setVgap(v);
    }
}  // public class GriddedPanel

