package gui;

import java.awt.Dimension;
import javax.swing.JLabel;

/**
 * A Spacer is an empty label that is sized according to
 * its current font size.
 * Its width and height are specified with em units in
 * the constructors and the setter methods setWidthEm(),
 * setHeightEm(), and setSizeEm().
 * The getter methods getMinimumSize(), getPreferredSize(),
 * and getMaximumSize() use pixels as units, as required
 * by layout managers.
 * These three methods all return the same value.
 * 
 * @author gshute
 */
public class Spacer
extends JLabel {
  
  private float heightEm;
  private float widthEm;
  
  /**
   * new Spacer(hEm) returns a Spacer with dimensions
   * 0 by hEm ems.
   * 
   * @param hEm the height in ems
   */
  public Spacer(float hEm) {
    setSizeEm(0, hEm);
  }
  
  /**
   * new Spacer(wEm, hEm) returns a Spacer with
   * dimensions wEm ems by hEm ems.
   * 
   * @param wEm the width in ems
   * @param hEm the height in ems
   */
  public Spacer(float wEm, float hEm) {
    widthEm = wEm;
    heightEm = hEm;
  }
  
  /**
   * sp.getMinimumSize() returns the minimum size
   * for sp.
   * The units in the returned dimension are pixels.
   * 
   * @return the minimum size in pixels
   */
  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  /**
   * sp.getPreferredSize() returns the preferred size
   * for sp.
   * The units in the returned dimension are pixels.
   * 
   * @return the preferred size in pixels
   */
  public Dimension getPreferredSize() {
    int preferredWidth =
            Math.round(widthEm*getFont().getSize2D());
    int preferredHeight =
            Math.round(heightEm*getFont().getSize2D());
    return new Dimension(preferredWidth, preferredHeight);
  }
  
  /**
   * sp.getMaximumSize() returns the maximum size
   * for sp.
   * The units in the returned dimension are pixels.
   * 
   * @return the maximum size in pixels
   */
  public Dimension getMaximumSize() {
    return getPreferredSize();
  }
  
  /**
   * sp.setWidthEm(hEm) sets the minimum,
   * preferred, and maximum widths of sp to
   * wEm ems.
   * The heights are unaffected.
   * 
   * @param wEm the width in ems
   */
  public void setWidthEm(float wEm) {
    widthEm = wEm;
  }
  
  /**
   * sp.setHeightEm(hEm) sets the minimum,
   * preferred, and maximum heights of sp to
   * hEm ems.
   * The widths are unaffected.
   * 
   * @param hEm the height in ems
   */
  public void setHeightEm(float hEm) {
    heightEm = hEm;
  }
  
  /**
   * sp.setSizeEm(wEm, hEm) sets the minimum,
   * preferred, and maximum sizes of sp to
   * wEm ems by hEm ems.
   * 
   * @param wEm the width in ems
   * @param hEm the height in ems
   */
  public void setSizeEm(float wEm, float hEm) {
    widthEm = wEm;
    heightEm = hEm;
  }

}  // public class Spacer
