package gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;

/**
 * The Wrapper class provides convenience static methods for
 * wrapping components and viewables in panels, scrollpanes,
 * and title bordered panels.
 * The methods all have variants that permit sizing of the
 * wrapped component or wrapped viewable.
 *
 * @author gshute
 */
public class Wrapper {

  private static WrapMethod paneledWrapMethod;
  private static WrapMethod unwrappedWrapMethod;

  /**
   * This constructor is disabled.
   * All class methods are static.
   */
  private Wrapper() {
  }

  /**
   * Wrapper.unwrapped(c) returns c as is.
   * 
   * @param c the component
   * @return the component c as is
   */
  public static JComponent unwrapped(JComponent c) {
    return c;
  }

  /**
   * Wrapper.unwrapped(c, w, h) returns c with its minimum,
   * preferred, and maximum sizes set to w by h.
   * @param c the component
   * @param w the width of the returned component
   * @param h the height of the returned component
   * @return the sized component c
   */
  public static <T extends JComponent> T unwrapped(T c, int w, int h) {
    Dimension dimension = new Dimension(w, h);
    c.setMinimumSize(dimension);
    c.setPreferredSize(dimension);
    c.setMaximumSize(dimension);
    return c;
  }

  /**
   * Wrapper.unwrapped(v) returns the view component of v as is.
   * 
   * @param v the viewable whose view component is to be returned
   * @return the view component of v
   */
  public static JComponent unwrapped(Viewable v) {
    return v.getViewComponent();
  }

  /**
   * Wrapper.unwrapped(v, w, h) returns the view component of v
   * with its minimum, preferred, and maximum sizes set to w by h.
   * 
   * @param v the viewable whose view component is to be returned
   * @param w the width of the returned component
   * @param h the height of the returned component
   * @return the sized view component of v
   */
  public static JComponent unwrapped(Viewable v, int w, int h) {
    return unwrapped(v.getViewComponent(), w, h);
  }

  /**
   * Wrapper.paneled(c) returns a panel containing c.
   * 
   * @param c the component to be wrapped
   * @return a panel containing c
   */
  public static JPanel paneled(JComponent c) {
    JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    panel.add(c);
    return panel;
  }

  /**
   * Wrapper.paneled(c, w, h) returns a sized panel containing c.
   * The minimum, preferred, and maximum sizes of the panel
   * are set to w by h.
   * 
   * @param c the component to be wrapped
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized panel containing c
   */
  public static JPanel paneled(JComponent c, int w, int h) {
    return unwrapped(paneled(c), w, h);
  }  // public static JPanel paneled(JComponent, int, int)

  /**
   * Wrapper.paneled(v) returns a panel containing the
   * view component of v.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @return a panel containing the view component of v
   */
  public static JPanel paneled(Viewable v) {
    return paneled(v.getViewComponent());
  }

  /**
   * Wrapper.paneled(v, w, h) returns a sized panel containing
   * the view component of v.
   * The minimum, preferred, and maximum sizes of the panel
   * are set to w by h.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized panel containing the view component of v
   */
  public static JPanel paneled(Viewable v, int w, int h) {
    return unwrapped(paneled(v), w, h);
  }

  /**
   * Wrapper.scrolled(c) returns a scroll pane containing the
   * view component of c.
   * 
   * @param c the component to be scrolled
   * @return a scroll pane containing c
   */
  public static JScrollPane scrolled(JComponent c) {
    return new JScrollPane(c,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
  }

  /**
   * Wrapper.scrolled(c, w, h) returns a sized scroll pane
   * containing c.
   * The minimum, preferred, and maximum sizes of the scroll
   * pane are set to w by h.
   * 
   * @param c the component to be scrolled
   * @param w the width of the returned scroll pane
   * @param h the height of the returned scroll pane
   * @return a sized scroll pane containing c
   */
  public static JScrollPane scrolled(JComponent c, int w, int h) {
    return unwrapped(scrolled(c), w, h);
  }

  /**
   * Wrapper.scrolled(v) returns a sized scroll pane
   * containing the view component of v.
   * The minimum, preferred, and maximum sizes of the scroll
   * pane are set to w by h.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @return a sized scroll pane containing c
   */
  public static JScrollPane scrolled(Viewable v) {
    return scrolled(v.getViewComponent());
  }

  /**
   * Wrapper.scrolled(v, w, h) returns a sized scroll pane
   * containing the view component of v.
   * The minimum, preferred, and maximum sizes of the scroll
   * pane are set to w by h.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @param w the width of the returned scroll pane
   * @param h the height of the returned scroll pane
   * @return a scroll pane containing c
   */
  public static JScrollPane scrolled(Viewable v, int w, int h) {
    return unwrapped(scrolled(v), w, h);
  }

  /**
   * Wrapper.titled(c, ttl) returns a scroll pane containing c.
   * The border is titled with the title ttl.
   * 
   * @param c the component to be wrapped
   * @param ttl the panel border title 
   * @return a scroll pane containing c
   */
  public static JPanel titled(JComponent c, String ttl) {
    JPanel panel = paneled(c);
    TitledBorder border = BorderFactory.createTitledBorder(ttl);
    panel.setBorder(border);
    Dimension borderDimension = border.getMinimumSize(panel);
    Insets insets = border.getBorderInsets(panel);
    Dimension componentDimension = c.getPreferredSize();
    int width = Math.max(borderDimension.width, componentDimension.width) + insets.right + 1;
    int height = borderDimension.height + componentDimension.height + insets.bottom;
    Dimension dimension = new Dimension(width, height);
    panel.setMinimumSize(dimension);
    panel.setPreferredSize(dimension);
    panel.setMaximumSize(dimension);
    return panel;
  }

  /**
   * Wrapper.titled(c, ttl, w, h) returns a sized title bordered panel
   * containing c.
   * The border is titled with the title ttl.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * @param c the component to be wrapped
   * @param ttl the panel border title 
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized title bordered panel containing c
   */
  public static JPanel titled(JComponent c, String ttl, int w, int h) {
    return unwrapped(titled(c, ttl), w, h);
  }

  /**
   * Wrapper.titled(v, ttl) returns a title bordered panel containing
   * the view component of v.
   * The border is titled with the title ttl.
   * @param v the viewable whose view component is to be wrapped
   * @param ttl the panel border title 
   * @return a title bordered panel containing c
   */
  public static JPanel titled(Viewable v, String ttl) {
    return titled(v.getViewComponent(), ttl);
  }

  /**
   * Wrapper.titled(v, ttl, w, h) returns a sized title bordered panel
   * containing the view component of v.
   * The border is titled with the title ttl.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @param ttl the panel border title 
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized title bordered panel containing the view component
   * of v
   */
  public static JPanel titled(Viewable v, String ttl, int w, int h) {
    return unwrapped(titled(v, ttl), w, h);
  }

  /**
   * Wrapper.named(c) returns a title bordered panel containing c.
   * The border is titled with the name of c.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * @param c the component to be wrapped
   * @return a title bordered panel containing c
   */
  public static JPanel named(JComponent c) {
    return titled(c, c.getName());
  }

  /**
   * Wrapper.named(c, w, h) returns a sized title bordered panel
   * containing c.
   * The border is titled with the name of c.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * 
   * @param c the component to be wrapped
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized title bordered panel containing c
   */
  public static JPanel named(JComponent c, int w, int h) {
    return titled(c, c.getName(), w, h);
  }

  /**
   * Wrapper.named(v) returns a title bordered panel containing
   * the view component of v.
   * The border is titled with the name of v's view component.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @return a title bordered panel containing c
   */
  public static JPanel named(Viewable v) {
    return titled(v, v.getViewComponent().getName());
  }

  /**
   * Wrapper.named(v, w, h) returns a sized title bordered panel
   * containing the view component of v.
   * The border is titled with the name of v's view component.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * 
   * @param v the viewable whose view component is to be wrapped
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized title bordered panel containing the view component
   * of v
   */
  public static JPanel named(Viewable v, int w, int h) {
    return unwrapped(named(v), w, h);
  }

  /**
   * Wrapper.named(nv) returns a title bordered panel containing
   * the view component of v.
   * The border is titled with the name of nv.
   * 
   * @param nv the named viewable whose view component is to be wrapped
   * @return a title bordered panel containing the view component of nv
   */
  public static JPanel named(NamedViewable nv) {
    return titled(nv, nv.getName());
  }

  /**
   * Wrapper.named(nv, w, h) returns a sized title bordered panel
   * containing the view component of v.
   * The border is titled with the name of nv.
   * The minimum, preferred, and maximum sizes of the panel are set to
   * w by h.
   * 
   * @param nv the named viewable whose view component is to be wrapped
   * @param w the width of the returned panel
   * @param h the height of the returned panel
   * @return a sized title bordered panel containing the view component
   * of nv
   */
  public static JPanel named(NamedViewable nv, int w, int h) {
    return titled(nv, nv.getName(), w, h);
  }

  /**
   * This method is experimental - it may be of little use.
   */
  public static WrapMethod createPaneledWrapMethod() {
    if (paneledWrapMethod == null) {
      paneledWrapMethod = new WrapMethod() {

        public JComponent getWrapComponent(Viewable v) {
          return paneled(v);
        }

      };
    }
    return paneledWrapMethod;
  }

  /**
   * This method is experimental - it may be of little use.
   */
  public static WrapMethod createUnwrappedWrapMethod() {
    if (unwrappedWrapMethod == null) {
      unwrappedWrapMethod = new WrapMethod() {

        public JComponent getWrapComponent(Viewable v) {
          return unwrapped(v);
        }

      };
    }
    return unwrappedWrapMethod;
  }

}  // public class Wrapper
