package gui.multisource;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.swing.JTextPane;
import javax.swing.border.AbstractBorder;
import javax.swing.plaf.TextUI;
import javax.swing.text.BadLocationException;

/**
 * A MultisourceBorder is a border that displays a
 * color code indicating the source of text in a
 * MultisourceDocument.
 * The border can be applied to a text pane that
 * contains a multisource document.
 * Its color codes appear to the left of the text.
 */
public class MultisourceBorder
        extends AbstractBorder {

  private JTextPane textPane;
  private MultisourceDocument document;
  private int leftInset;
  private Map<Object, Color> colorMap;

  private int getMinY(int off) {
    try {
      Rectangle bounds = textPane.modelToView(off);
      return bounds.y;
    } catch (BadLocationException ex) {
      System.out.println(ex.getMessage() + ": " + off);
    }
    return -1;
  }

  private int getMaxY(int off) {
    try {
      Rectangle bounds = textPane.modelToView(off);
      return bounds.y + bounds.height;
    } catch (BadLocationException ex) {
      System.out.println(ex.getMessage() + ": " + off);
    }
    return -1;
  }

  /**
   * new MultisourceBorder(c, msd) returns a multisource border
   * that reflects the sources for text in msd and can be installed
   * in c.
   * 
   * @param tp the text pane
   * @param msd the multisource document
   */
  public MultisourceBorder(
          JTextPane tp, MultisourceDocument msd) {
    this(tp, msd, 4);
  }

  /**
   * new MultisourceBorder(tp, msd, w) returns a new
   * multisource border with width w that reflects the
   * sources for text in msd and can be installed on tp.
   * 
   * @param tp the text pane
   * @param msd the multisource document
   */
  public MultisourceBorder(
          JTextPane tp, MultisourceDocument msd, int w) {
    textPane = tp;
    document = msd;
    leftInset = w + 4;
    colorMap = new HashMap<Object, Color>();
  }

  /**
   * msb.setSourceColor(s, c) sets the source color for
   * sources whose identifier is s to c.
   * 
   * @param s
   * @param c
   */
  public void setSourceColor(Object s, Color c) {
    colorMap.put(s, c);
  }

  /**
   * msb.getBorderInsets(c) returns the insets for msb
   * in c.
   * 
   * @param c the component in which the border is installed
   * @return the insets
   */
  public Insets getBorderInsets(Component c) {
    return getBorderInsets(c, new Insets(0, 0, 0, 0));
  }

  /**
   * msb.getBorderInsets(c, ins) copies the insets for msb
   * in c into .
   * 
   * @param c the component in which the border is installed
   * @param ins the insets object that receives the insets
   * @return the insets (ins)
   */
  public Insets getBorderInsets(Component c, Insets ins) {
    int li = textPane.equals(c) ? leftInset : 0;
    ins.set(0, li, 0, 0);
    return ins;
  }

  /**
   * The paintBorder() method is invoked by the component in
   * which this border is installed.
   * Other clients will not normally invoke this method.
   */
  public void paintBorder(Component c, Graphics g,
          int x, int y, int w, int h) {
    if (!textPane.equals(c)) {
      return;
    }
    if (x + 4 >= leftInset) {
      return;
    }
    int width = leftInset - 4;
    TextUI mapper = textPane.getUI();
    Point upperLeft = new Point(x + leftInset, y);
    int startOffset =
            mapper.viewToModel(textPane, upperLeft);
    Point lowerLeft = new Point(x + leftInset, y + h);
    int endOffset = mapper.viewToModel(textPane, lowerLeft);
    Iterator<MultisourceDocument.SourceSegment> iterator =
            document.segments(startOffset, endOffset);
    while (iterator.hasNext()) {
      MultisourceDocument.SourceSegment segment =
              iterator.next();
      int yMin = getMinY(segment.getStartOffset());
      int height =
              getMaxY(segment.getEndOffset() - 1) - yMin;
      Object source = segment.getSource();
      Color color = colorMap.get(source);
      if (color == null) {
        color = Color.BLACK;
      }
      g.setColor(color);
      g.fillRect(x, yMin, width, height);
    }
  }

}  // public class MultisourceBorder
