package gui.multisource;

import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.DocumentFilter;
import javax.swing.text.SimpleAttributeSet;

/**
 * A MultisourceDocument is a document with segments that
 * are attributed to different sources.
 * The sources can be indicated with a MultisourceBorder.
 * 
 */
public class MultisourceDocument
        extends DefaultStyledDocument {

  /**
   * MultisourceDocument.USER is the source value for
   * user entered text that has been committed.
   */
  public static final Object USER =
          new Object() {

            public String toString() {
              return "user";
            }

          };
  /**
   * MultisourceDocument.SOURCE_ATTTRIBUTE is the key for
   * the source attribute of text.
   */
  private static final Object SOURCE_ATTRIBUTE =
          new Object() {

            public String toString() {
              return "source";
            }

          };

  /**
   * A SourceSegment represents a contiguous, single
   * source segment of text in a multisource document.
   * It provides information about the start and end 
   * offsets of the segment and its source.
   */
  public class SourceSegment {

    private Object source;
    private int startOffset;
    private int endOffset;

    private SourceSegment(Object src, int so, int eo) {
      source = src;
      startOffset = so;
      endOffset = eo;
    }

    public Object getSource() {
      return source;
    }

    public int getStartOffset() {
      return startOffset;
    }

    public int getEndOffset() {
      return endOffset;
    }

    public String toString() {
      return "SourceSegment[" + source + ", " +
              startOffset + ", " + endOffset + "]";
    }

  }  // public class SourceSegment

  /**
   * A SegmentFilter is a document filter that updates
   * the segment structure when this multisource document
   * is edited.
   */
  private class SegmentFilter
          extends DocumentFilter {

    public void insertString(DocumentFilter.FilterBypass fb,
            int off, String txt, AttributeSet attr)
            throws BadLocationException {
      Object source = attr.getAttribute(SOURCE_ATTRIBUTE);
      if (source == null) {
        // If source is null then text is replaced by the user.
        doUserSourceReplace(fb, off, 0, txt, attr);
      } else {
        // Else text is added by invoking addText().
        doOtherSourceReplace(fb, off, 0, txt, source);
      }
    }

    public void remove(DocumentFilter.FilterBypass fb,
            int off, int len)
            throws BadLocationException {
      if (startNewText <= off) {
        fb.remove(off, len);
      }
    }

    public void replace(DocumentFilter.FilterBypass fb,
            int off, int len, String txt, AttributeSet attr)
            throws BadLocationException {
      Object source = attr.getAttribute(SOURCE_ATTRIBUTE);
      if (source == null) {
        // If source is null then text is replaced by the user.
        doUserSourceReplace(fb, off, len, txt, attr);
      } else {
        // Else text is added by invoking addText().
        doOtherSourceReplace(fb, off, len, txt, source);
      }
    }

  }  // private class SegmentFilter

  private TextConsumer textConsumer;
  private boolean autoCommit;
  private java.util.List<SourceSegment> segmentList;
  private int startNewText;
  private DefaultCaret caret;

  private void updateSegmentList(int eo, Object src) {
    SourceSegment segment = null;
    int segmentCount = segmentList.size();
    if (segmentCount > 0) {
      segment = segmentList.get(segmentCount - 1);
    }
    if (segment == null || segment.getSource() != src) {
      segment = new SourceSegment(src, startNewText, eo);
      segmentList.add(segment);
    } else {
      segment.endOffset = eo;
    }
    startNewText = eo;
  }

  private void doCommitUserText() {
    try {
      int endOffset = getLength();
      int textLength = endOffset - startNewText;
      final String text =
              getText(startNewText, textLength);
      updateSegmentList(endOffset, USER);
      SwingUtilities.invokeLater(new Runnable() {

        public void run() {
          textConsumer.consumeText(text);
        }

      });
    } catch (BadLocationException ex) {
    }
  }

  private void doUserSourceReplace(
          DocumentFilter.FilterBypass fb,
          int off, int len, String txt, AttributeSet attr)
          throws BadLocationException {
    if (startNewText <= off) {
      boolean hasNewline = false;
      if (autoCommit && txt.endsWith("\n")) {
        txt = txt.substring(0, txt.length() - 1);
        hasNewline = true;
      }
      fb.replace(off, len, txt, attr);
      if (hasNewline) {
        int endOffset = getLength();
        fb.replace(endOffset, 0, "\n", attr);
        doCommitUserText();
        caret.setDot(getLength());
      }
    }
  }

  private void doOtherSourceReplace(
          DocumentFilter.FilterBypass fb,
          int off, int len, String txt, Object src)
          throws BadLocationException {
    fb.replace(off, len, txt, null);
    updateSegmentList(off + txt.length() - len, src);
  }

  private int findIndex(int off) {
    if (off < 0) {
      return 0;
    }
    int minIndex = 0;
    int maxIndex = segmentList.size() - 1;
    while (minIndex <= maxIndex) {
      int midIndex = (minIndex + maxIndex) / 2;
      SourceSegment midSegment =
              segmentList.get(midIndex);
      int startOffset = midSegment.getStartOffset();
      int endOffset = midSegment.getEndOffset();
      if (startOffset > off) {
        maxIndex = midIndex - 1;
      } else if (off >= endOffset) {
        minIndex = midIndex + 1;
      } else { // startOffset <= off < endOffset
        return midIndex;
      }
    }
    return segmentList.size();
  }

  /**
   * new MultisourceDocument(tc) returns a new
   * MultisourceDocument with text consumer tc.
   */
  public MultisourceDocument(TextConsumer tc) {
    super();
    textConsumer = tc;
    autoCommit = true;
    segmentList = new ArrayList<SourceSegment>();
    startNewText = 0;
    caret = new DefaultCaret();
    setDocumentFilter(new SegmentFilter());
  }

  /**
   * msd.segments() returns an iterator for the segments
   * in msd that contain offsets from so to eo, inclusive.
   */
  public Iterator<SourceSegment> segments(int so, int eo) {
    int startIndex = findIndex(so);
    int endIndex = findIndex(eo);
    java.util.List<SourceSegment> sublist =
            segmentList.subList(startIndex, endIndex);
    return sublist.iterator();
  }

  /**
   * msd.getStartNewText() returns msd's start new text
   * offset.
   */
  public int getStartNewText() {
    return startNewText;
  }

  /**
   * msd.getCaret() returns msd's edit caret.
   */
  public DefaultCaret getCaret() {
    return caret;
  }

  /**
   * msd.setTextConsumer(tc) sets msd's text consumer
   * to tc.
   */
  public void setTextConsumer(TextConsumer tc) {
    textConsumer = tc;
  }

  /**
   * msd.commitUserText() commits the currently
   * uncommitted user text in msd and sends the text
   * to msd's text consumer.
   */
  public void commitUserText() {
    writeLock();
    try {
      doCommitUserText();
    } finally {
      writeUnlock();
    }
  }

  /**
   * msd.addText(txt, src) adds txt to msd with source
   * src.
   */
  public void addText(String txt, Object src) {
    if (src == null) {
      throw new IllegalArgumentException("src is null");
    }
    // Add newline if missing.
    if (txt.length() == 0 ||
            txt.charAt(txt.length() - 1) != '\n') {
      txt += "\n";
    }
    SimpleAttributeSet attributes =
            new SimpleAttributeSet();
    attributes.addAttribute(SOURCE_ATTRIBUTE, src);
    try {
      replace(startNewText, 0, txt, attributes);
    } catch (BadLocationException ex) {
    }
  }

  /**
   * msd.clear() removes all text from msd.
   */
  public void clear() {
    writeLock();
    try {
      segmentList.clear();
      startNewText = 0;
      remove(0, getLength());
    } catch (BadLocationException ex) {
    } finally {
      writeUnlock();
    }
  }

}  // public class MultisourceDocument
