package gui.multisource;

import gui.Viewable;
import java.awt.Color;
import java.awt.Font;
import javax.swing.JTextPane;
import javax.swing.text.NavigationFilter;
import javax.swing.text.Position;

/**
 * A MultisourceViewer displays multisource text and
 * displays the sources of lines of text as color codes
 * on the left of the text.
 * A multisource viewer has a special source identified
 * as MultisourceDocument.USER that represents the user
 * of the program.
 * The last line of the viewer can be freely edited by
 * the user.
 * It is committed automatically when the user enters a
 * linefeed (return) character.
 * When committed, the line is forwared to a text
 * consumer that is attached to the viewer and it color
 * code is set to the color code for USER.
 */
public class MultisourceViewer
        implements Viewable {

  private JTextPane textPane;
  private MultisourceDocument document;
  private MultisourceBorder margin;

  /**
   * createNavigationFilter() returns a navigation filter
   * that prevents moving the insertion caret before the
   * start-new-text point of this terminal pane.
   */
  private NavigationFilter createNavigationFilter() {
    return new NavigationFilter() {

      public void moveDot(NavigationFilter.FilterBypass fb,
              int dot, Position.Bias b) {
        if (dot < document.getStartNewText()) {
          dot = document.getStartNewText();
        }
        super.moveDot(fb, dot, b);
      }

      public void setDot(NavigationFilter.FilterBypass fb,
              int dot, Position.Bias b) {
        if (dot < document.getStartNewText()) {
          dot = document.getStartNewText();
        }
        super.setDot(fb, dot, b);
      }

    };
  }

  /**
   * new MultisourceViewer() returns a new MultisourceViewer
   * with a do-nothing text consumer.
   */
  public MultisourceViewer() {
    this(new TextConsumer() {

      public void consumeText(String txt) {
      }

    });
  }

  /**
   * new MultisourceViewer(tc) returns a new MultisourceViewer
   * with text consumer tc.
   */
  public MultisourceViewer(TextConsumer tc) {
    document = new MultisourceDocument(tc);
    textPane = new JTextPane(document);
    textPane.setCaret(document.getCaret());
    margin = new MultisourceBorder(textPane, document, 8);
    textPane.setBorder(margin);
    textPane.setEditable(true);
    textPane.setNavigationFilter(createNavigationFilter());
    margin.setSourceColor(MultisourceDocument.USER, Color.BLUE);
  }

  /**
   * mv.getViewComponent() returns the text pane for mv.
   */
  public JTextPane getViewComponent() {
    return textPane;
  }

  /**
   * mv.setFontSize(sz) sets the font size of mv to sz.
   */
  public void setFontSize(int sz) {
    textPane.setFont(new Font("Monospaced", Font.PLAIN, sz));
  }

  /**
   * mv.setSourceColor(src, c) sets the source color of
   * source identified by src in mv to c.
   */
  public void setSourceColor(Object src, Color c) {
    margin.setSourceColor(src, c);
  }

  /**
   * mv.setTextConsumer(tc) sets tc as the text consumer
   * for mv.
   */
  public void setTextConsumer(TextConsumer tc) {
    document.setTextConsumer(tc);
  }

  /**
   * mv.setEditable(ed) sets the editability of mv to ed.
   */
  public void setEditable(final boolean ed) {
    textPane.setEditable(ed);
  }

  /**
   * mv.addText(s, src) adds s at the start-new-text point of mv
   * and advances the start-new-text point past the added text.
   * The source of the text is indicated by src.
   * The text is added by the event handler thread, allowing the
   * method to be invoked by other threads.
   */
  public void addText(final String s, final Object src) {
    javax.swing.SwingUtilities.invokeLater(new Runnable() {

      public void run() {
        document.addText(s, src);
      }

    });
  }

  /**
   * mv.clear() removes all text from mv.
   */
  public void clear() {
    document.clear();
  }

}  // public class MultisourceViewer
