/*
 * Choice.java
 *
 * Created on August 18, 2006, 9:33 AM
 *
 */
package mvc;

import gui.Viewable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.swing.JComponent;
import conversion.Converter;

/**
 * A Choice<T> is a model type that restricts model values
 * to a fixed set of choices with Java type T.
 * A Choice<T> is a factory class for creating models.
 * It also defines a control protocol for its models.
 *
 * @author gshute
 */
public class Choice<T>
        implements Iterable<T> {

  /**
   * This is a relay method that allows us to keep getValueIndex()
   * private in Choice.Model.
   */
  private static <TD> int getValueIndex(Choice.Model<TD> m) {
    return m.getValueIndex();
  }

  /**
   * This is a relay method that allows us to keep setValueIndex()
   * private in Choice.Model.
   */
  private static <TD> void setValueIndex(Choice.Model<TD> m, int indx) {
    m.setValueIndex(indx);
  }

  /**
   * DOCME.
   */
  public static class Model<TM>
          extends NamedDirectModel<TM> {

    Choice<TM> choice;
    int valueIndex;

    /**
     * new Choice.Model(nm, c) returns a new model
     * named nm with choices from c.
     */
    private Model(String nm, Choice<TM> c) {
      super(nm);
      choice = c;
      valueIndex = 0;
    }

    /**
     * m.getValueIndex() returns the index of the current
     * value of m.
     * @return 
     */
    private int getValueIndex() {
      return valueIndex;
    }

    /**
     * m.setValueIndex(indx) sets the value of m to the
     * value indexed by indx.
     * @param indx 
     */
    private void setValueIndex(int indx) {
      valueIndex = indx;
      notifyObservers();
    }

    /**
     * m.getValue returns the current value of m.
     * @return 
     */
    public TM getValue() {
      return choice.getValue(valueIndex);
    }

    /**
     * m.getChoice() returns the choice of values for m.
     * @return 
     */
    public Choice<TM> getChoice() {
      return choice;
    }

  }  // public static class Model<TM>

  /**
   *
   */
  public static interface Controller<T>
          extends Viewable {

    /**
     * DOCME.
     * @param m
     */
    public void setModel(Choice.Model<? extends T> m);

    /**
     * DOCME.
     * @return
     */
    public Choice.Model<? extends T> getModel();

  }  // public static interface Controller<T>

  /**
   * Concrete subclasses should retain TC as a parameter but should
   * replace TD in the class declaration and all of its method
   * declarations with a non-generic class that can be displayed by
   * the subclass.
   */
  public static abstract class ControllerImplementation<TC, TD>
          implements Controller<TC> {

    private JComponent viewComponent;
    private Converter<? super TC, ? extends TD> converter;
    private UpdateManager<TC, Choice.Model<? extends TC>> updateManager;

    /**
     * displayObjects() returns an iterator for the displayObjects
     * of the choices for this editor.
     */
    private Iterator<TD> displayObjects() {
      return new Iterator<TD>() {

        private Iterator<? extends TC> values =
                getModel().getChoice().iterator();

        public boolean hasNext() {
          return values.hasNext();
        }

        public TD next() {
          return converter.convertTo(values.next());
        }

        public void remove() {
          throw new UnsupportedOperationException();
        }

      };
    }

    protected abstract JComponent createViewComponent();

    /**
     * Concrete subclasses should implement setDisplayObjects() to
     * record and display objects from nms.
     * The objects in nms are in view index order.
     * @param nms 
     */
    protected abstract void setDisplayObjects(Iterator<TD> nms);

    /**
     * DOCME.
     * @param indx 
     */
    protected abstract void setViewIndex(int indx);

    /**
     * DOCME.
     * @return 
     */
    protected abstract int getViewIndex();

    /**
     * DOCME.
     */
    protected final void setModelValue() {
      updateManager.setModelValue();
    }

    /**
     * new Controller(m, cvtr) returns a new Controller for the
     * values of m.
     * The values are converted to display objects by cvtr.
     * @param m 
     * @param cvtr 
     */
    public ControllerImplementation(Choice.Model<? extends TC> m,
            Converter<? super TC, ? extends TD> cvtr) {
      viewComponent = createViewComponent();
      Callback updateModelResponder = new Callback() {

        public void execute() {
          setValueIndex(getModel(), getViewIndex());
        }

      };
      Callback updateViewResponder = new Callback() {

        public void execute() {
          setViewIndex(getValueIndex(getModel()));
        }

      };
      Callback replaceModelResponder = new Callback() {

        public void execute() {
          setDisplayObjects(displayObjects());
        }

      };
      updateManager = new UpdateManager<TC, Choice.Model<? extends TC>>(m,
              updateModelResponder, replaceModelResponder, updateViewResponder);
      converter = cvtr;
      setModel(m);
    }

    /**
     * DOCME.
     * @return
     */
    public JComponent getViewComponent() {
      return viewComponent;
    }

    /**
     * DOCME.
     * @param m
     */
    public void setModel(Choice.Model<? extends TC> m) {
      updateManager.setModel(m);
    }

    /**
     * DOCME.
     * @return
     */
    public Choice.Model<? extends TC> getModel() {
      return updateManager.getModel();
    }

  }  // public static abstract class ControllerImplementation<TC, TD>

  List<T> choices;
  // It may be desirable to add some two-parameter constructors where the
  // first parameter is a default choice and the second parameter is an
  // array, an iterator, or a container for the non-default choices.
  /**
   * new Choice<T>(cl) returns a new Choice<T>
   * with choice values from cl.
   * Note that the entry type of cl must exactly match T.
   * @param cl 
   */
  public Choice(T[] cl) {
    this(Arrays.asList(cl));
  }

  /**
   * new Choice<T>(cl) returns a new Choice<T>
   * with choice values from cl.
   * @param cl 
   */
  public Choice(Iterable<? extends T> cl) {
    this(cl.iterator());
  }

  /**
   * new Choice<T>(cl) returns a new Choice<T>
   * with choice values from cl.
   * @param cl 
   */
  public Choice(Iterator<? extends T> cl) {
    choices = new ArrayList<T>();
    while (cl.hasNext()) {
      choices.add(cl.next());
    }
  }

  /**
   * mc.getValueCount() returns the number of choices in mc.
   * @return the count of choice values in mc
   */
  public int getValueCount() {
    return choices.size();
  }

  /**
   * mc.getValue(indx) returns the choice of mc indexed by indx.
   * @param indx the index for the value
   * @return the value indexed by indx in mc
   */
  public T getValue(int indx) {
    return choices.get(indx);
  }

  /**
   * mc.iterator() returns an iterator for the choices of mc.
   * @return an iterator for the choice values of mc
   */
  public Iterator<T> iterator() {
    return choices.iterator();
  }

  /**
   * mc.createModel(nm) returns a new editable model with name nm
   * whose possible values are the choices of mc.
   * @param nm the name for the returned model
   * @return a new model with name nm and choices from mc
   */
  public Choice.Model<T> createModel(String nm) {
    return new Choice.Model<T>(nm, this);
  }

}  // public class Choice<T>
