package vfs;

import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

/**
 * A JarDirectory is a VFSDirectory representing a
 * directory in a jar file.
 *
 * @author gshute
 */
public class JarDirectory
        extends VFSDirectory {

  /**
   * addSubdirectory(nm) adds the subdirectory named nm
   * to this JarDirectory or its descendents.
   */
  private void addSubdirectory(String nm) {
    int slashPosition = nm.indexOf("/");
    if (slashPosition < 0) {
      throw new IllegalArgumentException(
              "Illegal subdirectory name: " + nm);
    }
    if (slashPosition == nm.length() - 1) {
      JarDirectory newDirectory = new JarDirectory(nm, this);
      addSubdirectory(newDirectory);
    } else {
      String childName = nm.substring(0, slashPosition + 1);
      JarDirectory child =
              (JarDirectory) getSubdirectory(childName);
      child.addSubdirectory(nm.substring(slashPosition + 1));
    }
  }

  /**
   * addJarredFiles(url) adds all of the jar entries
   * below the jar file entry designated by url to this
   * JarDirectory.
   */
  private void addJarredFiles(URL url)
          throws IOException {
    JarFile jarFile =
            ((JarURLConnection) url.openConnection()).getJarFile();
    String path = url.getPath();
    int prefixStart = path.indexOf("!/") + 2;
    String prefix = path.substring(prefixStart);
    int prefixLength = prefix.length();
    Enumeration<JarEntry> enumeration = jarFile.entries();
    while (enumeration.hasMoreElements()) {
      JarEntry jarEntry = enumeration.nextElement();
      String name = jarEntry.getName();
      if (name.startsWith(prefix) && name.length() > prefixLength) {
        if (name.endsWith("/")) {
          String subdirectoryName = name.substring(prefixLength);
          addSubdirectory(subdirectoryName);
        } else {
          addFile(name.substring(prefixLength));
        }
      }
    }
  }

  /**
   * new JarDirectory(url) returns a new virtual file
   * system root directory representing the jar directory
   * whose URL is url.
   * 
   * @param url the URL of the jar root directory
   */
  public JarDirectory(URL url) {
    super(url);
    try {
      addJarredFiles(url);
    } catch (IOException ex) {
      throw new VFSError(ex);
    }
  }

  /**
   * new JarDirectory(nm, par) returns a virtual directory
   * named nm whose parent is par.
   * 
   * @param nm the name of the jar directory
   * @param par its virtual file system parent
   */
  public JarDirectory(String nm, JarDirectory par) {
    super(nm, par);
  }

}  // public class JarDirectory
