package vfs;

/**
 * A VFSVisitor executes an algorithm with
 * files and directories while being escorted
 * through a virtual file system.
 * <p>
 * This is a Visitor interface in a Visitor
 * design pattern whose ObjectStructure class
 * is a VFSRoot.
 * In addition to normal Visitor methods, it
 * adds methods start() and finish().
 * The start() method allows a VFSVisitor to
 * perfom multiple traversals through different
 * virtual file systems.
 * The finish() method() can be implemented to
 * perform any algorithm steps that are required
 * after all files and subdirectories have been
 * visited.
 * </p>
 * 
 * @author gshute
 */
public interface VFSVisitor {
  
  /**
   * v.start() performs algorithm initialization
   * steps required when a new traversal is
   * started with v.
   */
  public void start();

  /**
   * v.enter(d) performs algorithm steps required
   * when v is escorted into the directory d.
   * 
   * @param d the directory being entered
   */
  public void enter(VFSDirectory d);

  /**
   * v.leave(d) performs algorithm steps required
   * when v is escorted out of the directory d.
   * 
   * @param d the directory being left
   */
  public void leave(VFSDirectory d);

  /**
   * v.visit(f) performs algorithm steps required
   * when v is escorted to the file f.
   * 
   * @param f the file being visited
   */
  public void visit(VFSFile f);
  
  /**
   * v.finish() performs algorithm steps required
   * when a traversal with v is finished.
   */
  public void finish();
  
}  // public interface VFSVisitor
